//+------------------------------------------------------------------+
//|                                                 CloseAtPrice.mq4 |
//|                                                                  |
//|Written  by A.M.Meldrum to close at requested prices              |
//|With thanks to Metatrader 4 forum for some coding snipets         |
//+------------------------------------------------------------------+
#include <WinUser32.mqh> //add if using RemoveEA

extern string comment0="NOTE Logical Errors stop the EA" ;
extern string comment1="Is Trade to watch a Buy or Sell?" ;
extern string BuyOrSell="BUY" ;
extern string comment2="Enter Ticket ID if you have one." ;
extern int    Ticket = 0 ;
extern string comment3="To use pip values only Enter true or false" ;
extern bool   UsePips = true ;
extern string comment4="1 pip = smallest change in curency pair" ;
extern int    Pips_TP=10 ;
extern int    Pips_SL=10 ;
extern string comment5="Exact Prices to Close at; UsePips=false" ;
extern double takeprofit=0.0 ;
extern double stoploss=0.0 ;

static int    OrdType=-1 ;
static double OpPrice=0.0 ;

//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
  {       
   BuyOrSell=ToUpperCase(BuyOrSell) ; //Get the type of order EA works on
   if (StringFind(BuyOrSell,"BUY") >= 0) OrdType=OP_BUY ;
   if (StringFind(BuyOrSell,"SELL") >= 0) OrdType=OP_SELL ;
   if (StringFind(BuyOrSell,"LONG") >= 0) OrdType=OP_BUY ;
   if (StringFind(BuyOrSell,"SHORT") >= 0) OrdType=OP_SELL ;
   //EA entry errorchecks
   if (OrdType<0) 
     {             
      RemoveEA("Order Type to work on not found") ;//<< add ,false,false to stop Alert(msg) and Print(msg) 
     }
     
   if (UsePips == true)
     {
      if ( (Pips_SL <= 0) || (Pips_TP <= 0) ) 
        {           
         RemoveEA("Negative or Zero SL/TP is not supported") ;
        }
     }  
     
   //if ticket not 0 is there a ticket of this value if not then close EA
   if ( (Ticket != 0) && (OrderSelect(Ticket,SELECT_BY_TICKET,MODE_TRADES) == false) )
     {      
      RemoveEA("Ticket ",Ticket," cannot be selected") ;
     }
      
   //if no trade type BuyorSell in openorders list exit
   Ticket=GetTicket() ;
   if (Ticket < 0)     
     { 
      RemoveEA("Negative or Zero SL/TP is not supported") ;
     }
   else //check entered values work for selected order 
     {
      OpPrice = OrderOpenPrice() ;
      if (UsePips == false)
        {
         if (IsInRange(OpPrice,OrdType) == false)
           {             
            RemoveEA("Order Open Price outside Starts outside TakeProfit StopLoss limits") ;
           }          
        }
      else //ticket ok and UsePips is true so
        {
         if (OrdType == OP_BUY)  {takeprofit = OpPrice + Pips_TP*Point;} //set the EA test values for UsePips==true
         if (OrdType == OP_BUY)  {stoploss   = OpPrice - Pips_SL*Point;}
         if (OrdType == OP_SELL) {takeprofit = OpPrice - Pips_TP*Point;}
         if (OrdType == OP_SELL) {stoploss   = OpPrice + Pips_SL*Point;}
        }// end if (UsePips==false)    
     }// end else block of if ( Ticket<0)
   return(0) ;
  } // end int init()
//+------------------------------------------------------------------+
//| script "close first market order if it is first in the list"     |
//+------------------------------------------------------------------+
int start()
  {
   int error=0 ;
   string endmsg="" ;
   double temp ;    
   Comment("Watching Ticket ",Ticket," to Take Profit at ",takeprofit," or Stop Loss at ",stoploss) ;
   if ( (OrderSelect(Ticket,SELECT_BY_TICKET,MODE_TRADES) == false) )
     {
      RemoveEA("Ticket ",Ticket," Now Closed - EA removed from chart") ;
     }
     
   temp=OrderProfit() ;
   
   if ( IsInRange(OpPrice,OrdType) == false ) //price has moved above or below limits
     {   
      if ( OrderClose(OrderTicket(),OrderLots(),OrderClosePrice(),0,CLR_NONE) ) 
        {          
         if (temp > 0)  {endmsg="Order Closed on takeprofit";}
         if (temp < 0)  {endmsg="Order Closed on stoploss";}
         if (temp == 0) {endmsg="Order Closed at break even";}         
         RemoveEA(endmsg) ;
        }
      else
        {
         error=GetLastError(); Print("LastError = ",error);                  
        }                                                 
     }
   return(0);
  }
//+------------------------------------------------------------------+  
void RemoveEA(string msg, bool DoAlert=true, bool DoPrint=true) //Don't forget to use #include <WinUser32.mqh> for this function
 {
  if( DoAlert ) Alert(msg) ;
  if( DoPrint ) Print(msg ) ;
  int h = WindowHandle(Symbol(), Period());
  if (h != 0) PostMessageA(h, WM_COMMAND, 33050, 0);
 }
//+------------------------------------------------------------------+
int GetTicket()
{
 int result=-1 ;
 int i,active ;
 
 active = OrdersTotal() ;
 for (i=active-1; i>=0; i--)
    {
     if ( OrderSelect(i,SELECT_BY_POS,MODE_TRADES) )
       {
        if ( OrdType == OrderType() )
          {
           result=OrderTicket() ;
           break ;           
          }
       }
    }  
 return(result) ;
}
//+------------------------------------------------------------------+
string ToUpperCase(string sText)  //with thanks to thread http://forum.mql4.com/4891
{
  int iLen=StringLen(sText), i, iChar;
  for(i=0; i < iLen; i++) 
  {
    iChar=StringGetChar(sText, i);
    if(iChar >= 97 && iChar <= 122) sText=StringSetChar(sText, i, iChar-32);
  }
  return(sText);
}
//+------------------------------------------------------------------+
bool IsInRange( double OP, int OT )
{ //OP = OrderOpenPrice OT=OrderType ND = NormalizeDouble with Digits
 if ((  
      (OT == OP_BUY) &&            //Buy and
      (ND(takeprofit) >= OP) &&    //takeprotit > OrderOpenPrice and
      (OP >= ND(stoploss))         //OrderOpenPrice > stoploss 
     ) 
       ||                          //OR
     ( 
      (OT == OP_SELL) &&           //Sell and
      (ND(takeprofit) <= OP) &&    //takeprofit < OrderOpenPrice and
      (OP <= ND(stoploss))         //OrderOpenPrice < stoploss
     )
    )
  { 
   return( true ) ;
  }
  else
   return( false ) ;                    
}
//+------------------------------------------------------------------+
double ND( double var )
{
 return(NormalizeDouble(var,Digits)) ;
}